@@ -3,7 +3,7 @@ package Shipwright::Source::SVN;
 use warnings;
 use strict;
 use Carp;
-use File::Spec;
+use File::Spec::Functions qw/catfile catdir/;
 
 use base qw/Shipwright::Source::Base/;
 
@@ -16,11 +16,6 @@ sub new {
     my $self  = $class->SUPER::new(@_);
 
     $self->name( $self->just_name( $self->source ) ) unless $self->name;
-    $self->_update_url( $self->name, $self->source );
-
-    my $s = $self->source;
-    $s =~ s{^\s*svn:(?!//)}{};
-    $self->source($s);
     return $self;
 }
 
@@ -31,9 +26,11 @@ sub new {
 sub run {
     my $self = shift;
     $self->log->info( "prepare to run source: " . $self->source );
+    $self->_update_url( $self->name, 'svn:' . $self->source );
+
     $self->_run;
     my $s;
-    if ( $self->_is_compressed ) {
+    if ( $self->is_compressed ) {
         require Shipwright::Source::Compressed;
         $s = Shipwright::Source::Compressed->new( %$self, _no_update_url => 1 );
     }
@@ -51,19 +48,25 @@ sub run {
 sub _run {
     my $self   = shift;
     my $source = $self->source;
+
+
     my $cmd    = [
         'svn', 'export', $self->source,
-        File::Spec->catfile( $self->download_directory, $self->name )
+        catfile( $self->download_directory, $self->name ),
+        $self->version ? ( '-r', $self->version ) : (),
     ];
-    $self->source(
-        File::Spec->catfile( $self->download_directory, $self->name ) );
-    Shipwright::Util->run($cmd);
-}
 
-sub _is_compressed {
-    my $self = shift;
-    return 1 if $self->source =~ m{.*/(.+)\.(tar.(gz|bz2)|tgz)$};
-    return;
+    unless ( $self->version ) {
+        my ($out) = Shipwright::Util->run( [ 'svn', 'info', $source, ] );
+
+        if ( $out =~ /^Revision: (\d+)/m ) {
+            $self->version($1);
+        }
+    }
+
+    $self->source(
+        catfile( $self->download_directory, $self->name ) );
+    Shipwright::Util->run($cmd);
 }
 
 1;
